package actinScope;
//
//  CLASS
//    ActinScope  -
//		main class for simulation of dendritic actin filament
//
//
//  This class extends "Example.java" which, along with "ViewerBehavior, WalkViewerBehavior, and ExamineViewerBehavior
//  is taken from the work of David R. Nadeau / San Diego Supercomputer Center
//  Those classes take care of the virtual reality mouse control and Java3D universe setup
//
/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.image.*;
import com.sun.j3d.utils.geometry.*;
import javax.swing.*;
import java.io.*;
import infoCCD.*;

public class ActinScope extends Example {
	//--------------------------------------------------------------
	private static Object safeO = new Object();	// for synchronization of methods
	private Background background = null;
	private static Group scene = null;
	private static BranchGroup bugState = null;
	
	
	// the virtual world dimensions
	static final Pt3D worldDimension = new Pt3D(1,1,1);
	//static double worldScale = .1;
	//static double worldScaleInc = 0.01;
	
	static double deltaT = 0.0001;		// time-step
	static double simulationTime = 0;		// keeps track of current time
	static double runTime = 10000.0;		// final time to run to
	static int counter = 0;				// keeps track of current integration step number
	
	
	// for menus
	private static boolean paintOn = true;
	private CheckboxMenuItem paintOnMenuItem = null;
	private static CheckboxMenuItem stereoViewOnMenuItem = null;
	private static CheckboxMenuItem stereoViewCrossMenuItem = null;
	private static CheckboxMenuItem stereoViewDivergeMenuItem = null;
	private static boolean toFile = false;
	private static CheckboxMenuItem writeFileMenuItem = null;
	private static boolean paused = true;
	private static CheckboxMenuItem pausedMenuItem,runMenuItem;
	private static boolean viewRotation = false;
	private static CheckboxMenuItem rotateMenuItem = null;
	private static boolean boxShaded = true;
	private static CheckboxMenuItem boxShadedMenuItem = null;
	private static CheckboxMenuItem bugWiredMenuItem = null;
	private static CheckboxMenuItem filLinesMenuItem = null;
	private static CheckboxMenuItem filCylsMenuItem = null;
	private static CheckboxMenuItem monSpheresMenuItem = null;
	private static CheckboxMenuItem monDotsMenuItem = null;
	private CheckboxMenuItem lowQualityMenuItem,midQualityMenuItem,highQualityMenuItem;
	
	// for external frame menus/controls
	public RunControlFrame runControl;
	public RatesControlFrame ratesControl;
	public ConcControlFrame concControl;
	public InitControlFrame initControl;
	public MechControlFrame mechControl;
	
	// for graphics
	static int toScreenInterval = Env.toScreenInterval;
	static int toFileInterval = Env.toFileInterval;
	static boolean paintedThisStep;
	static String toFileName;

	public ActinScope () {
		super();
		new Info(true,true,false);	// make the InfoCCD materials
		runControl = new RunControlFrame(this);
		ratesControl = new RatesControlFrame(this);
		concControl = new ConcControlFrame(this);
		initControl = new InitControlFrame(this);
		mechControl = new MechControlFrame(this);
	}
	public void doLoop() {
		startUpdateTimer();
		int toScreenCounter = 0;
		int toFileCounter = 0;
		updateBugScene();		// draw once at the beginning of the simulation
		initializeRunControl(); // perform some necessary initialization of the runControl gui
		while (simulationTime <= runTime) {
			synchronized(safeO) {
				if ((paused) & (viewRotation)) { 
					rotateView();  // let's us set up a good looking rotation before starting movie
					try { Thread.sleep(20); } catch (InterruptedException e) { talkln ("error sleeping"); }
				}
				if (!paused) {
					for (int i=0;i<Thing.thingCt;i++) {
						Thing.theThings[i].step();
					}
					
					//update counters and flags
					paintedThisStep = false;
					counter++;
					simulationTime += deltaT;
					toScreenCounter++;
					toFileCounter++;
					
					if ((toScreenCounter >= toScreenInterval) & (paintOn) & (!toFile)) {
						if (viewRotation) { rotateView(); }
						updateBugScene();
						paintedThisStep = true;
						toScreenCounter = 0;
						try { Thread.sleep(20); } catch (InterruptedException e) { talkln ("error sleeping"); }
					}
					if ((toFileCounter >= toFileInterval) & (toFile)) {
						if (!paintedThisStep) {
							if (viewRotation) { rotateView(); }
							updateBugScene();
							paintedThisStep = true;
							try { Thread.sleep(100); } catch (InterruptedException e) { talkln ("error sleeping"); }
						}
						if (Env.stereoView) {
							captureImage(exCanvasL);
							captureImage(exCanvasR);
							try { Thread.sleep(3000); } catch (InterruptedException e) { talkln ("error sleeping"); }
							waitOnStereoCapture(exCanvasL,exCanvasR);
						} else {
							captureImage(exCanvasL);
							waitOnCapture(exCanvasL);
						}
						toFileCounter = 0;
						try { Thread.sleep(20); } catch (InterruptedException e) { talkln ("error sleeping"); }
					}
					// remove Things AFTER the graphical update... can detach graphics objects of dead things this way
					Thing.removeDeadThings();		// get rid of dead things
				} else {
					try { Thread.sleep(20); } catch (InterruptedException e) { talkln ("error sleeping"); }
				}
			}
		}
		System.exit(0);

	}
	
	public void startUpdateTimer() {
	  int delay = 2000; //milliseconds
	  ActionListener taskPerformer = new ActionListener() {
	      public void actionPerformed(ActionEvent evt) {
	    	  runControl.setInfoFields();
	    	  concControl.setInfoFields();
	      }
	  };
	  new Timer(delay, taskPerformer).start();
	}
	
	private BranchGroup initialBugScene() {
		bugState = new BranchGroup();
		allBranchGroupCapabilities(bugState);
		
		for (int i=0;i<Thing.thingCt;i++) {
			bugState.addChild(Thing.theThings[i].getGraphicsNode());
		}
		return bugState;
	}
	
	private void updateBugScene() {
		Thing curThing;
		for (int i=0;i<Thing.thingCt;i++) {
			Thing.theThings[i].advanceGraphics(bugState);
		}
	}
	
	private void initializeRunControl () {
		runControl.setLocation(0,Env.frameHeight+30);
		//runControl.setVisible(true);
		runControl.setCell(Thing.theCell);	// initialize cell reference in runControl
		runControl.setInfoFields();	// initialize the concentration fields
	}
	
	public void redoRun () {
		synchronized(safeO) {
			Filament.removeAll();
			Thing.removeDeadThings();
			Thing.thingCt = 0;	// no Things left
			Thing.theCell.G.detach();
			Thing.theCell = new Cell(new Pt3D(0,0,0),Env.cellX.getValue(),Env.cellY.getValue(),Env.cellZ.getValue());
			bugState.addChild(Thing.theCell.getGraphicsNode());
			concControl.lastValueAll();
			Filament.makeInitialFilaments();
			updateBugScene();
			//Thing.theCell.initializePistonPosition();
			//updateBugScene();
			concControl.refreshAll();
			ratesControl.refreshAll();
			simulationTime = 0;
		}
	}
	
	public static void captureImage (CapturingCanvas3D theCanvas) {
		theCanvas.writeJPEG_ = true;
		theCanvas.repaint();
	}
	
	public static void waitOnCapture (CapturingCanvas3D theCanvas) {
		while (!theCanvas.doneWithCapture()) {
			try { Thread.sleep(200); } catch (InterruptedException e) { talkln ("error sleeping"); }
		}
	}
	
	public static void waitOnStereoCapture (CapturingCanvas3D c1, CapturingCanvas3D c2) {
		int i=0;
		int sleepMillis = 200;
		while ((!c1.doneWithCapture()) & (!c2.doneWithCapture())) {
			talkln ("Waiting for stereo image capture ... " + String.valueOf(i*sleepMillis) + " milliseconds");
			try { Thread.sleep(sleepMillis); } catch (InterruptedException e) { talkln ("error sleeping"); }
			i++;
		}
	}
	
	
	//
	//  Build scene
	//
	public Group buildScene( )
	{
		
		// Build the scene root
		scene = new Group( );
		scene.setCapability(scene.ALLOW_CHILDREN_WRITE);
		scene.setCapability(scene.ALLOW_CHILDREN_EXTEND);

		// Create application bounds
		BoundingSphere worldBounds = new BoundingSphere(
			new Point3d( 0.0, 0.0, 0.0 ),  // Center
			1000.0 );                      // Extent

		// Set the background color and its application bounds
		background = new Background( );
		background.setColor(Env.universeColor3f);
		background.setCapability( Background.ALLOW_IMAGE_WRITE );
		background.setApplicationBounds( worldBounds );
		scene.addChild( background );

		// Build foreground geometry
		scene.addChild( initialBugScene( ) );

		return scene;
	}



	//--------------------------------------------------------------
	//  USER INTERFACE
	//--------------------------------------------------------------

	//
	//  Main
	//
	public static void main( String[] args )
	{
		ActinScope ex = new ActinScope( );
		ex.initialize( args );
		ex.buildUniverse( );
		ex.showFrame( );
		
		// make the instance of Rdm to create random arrays
		new Rdm(deltaT);
		
		// make Things
		// make the bug... only one and it's stationary
		Thing.theCell = new Cell(new Pt3D(0,0,0),Env.cellX.getValue(),Env.cellY.getValue(),Env.cellZ.getValue());
		Filament.makeInitialFilaments();
		
		ex.doLoop();
	}



	//
	//  Initialize the GUI (application and applet)
	//
	public void initialize( String[] args )
	{
		// Initialize the window, menubar, etc.
		super.initialize( args );
		exampleFrame.setTitle( "ActinScope - an actin simulation" );

	
		// Add menu for program control
		Menu m = new Menu ("Program");
		m.addActionListener(this);
		
		runMenuItem = new CheckboxMenuItem ("Run",!paused);
		runMenuItem.addItemListener(this);
		m.add(runMenuItem);
		
		pausedMenuItem = new CheckboxMenuItem ("Pause",paused);
		pausedMenuItem.addItemListener(this);
		m.add(pausedMenuItem);
		
		m.add("Redo");
		
		m.addSeparator();
		
		writeFileMenuItem = new CheckboxMenuItem("Write Files",toFile);
		writeFileMenuItem.addItemListener(this);
		m.add(writeFileMenuItem);
		
		exampleMenuBar.add(m);
		
		// Add a menu to select among options for painting etc
		m = new Menu( "View" );
		m.addActionListener(this);
		m.add("---General---");
		
		m.add("Reset View / Resize");
		
		paintOnMenuItem = new CheckboxMenuItem("Paint On",paintOn);
		paintOnMenuItem.addItemListener(this);
		m.add(paintOnMenuItem);
		
		headlightMenuItem = new CheckboxMenuItem( "Headlight on/off",true);
		headlightMenuItem.addItemListener( this );
		m.add( headlightMenuItem );
		
		rotateMenuItem = new CheckboxMenuItem("Rotate",viewRotation);
		rotateMenuItem.addItemListener(this);
		m.add(rotateMenuItem);
		
		Menu stereoView = new Menu ("Stereo View");
		
		stereoViewOnMenuItem = new CheckboxMenuItem ("On",Env.stereoView);
		stereoViewOnMenuItem.addItemListener(this);
		stereoView.add(stereoViewOnMenuItem);
		
		stereoView.addSeparator();
		
		stereoViewCrossMenuItem = new CheckboxMenuItem("Eyes Crossed",Env.eyesCrossed);
		stereoViewCrossMenuItem.addItemListener(this);
		stereoView.add(stereoViewCrossMenuItem);
		
		stereoViewDivergeMenuItem = new CheckboxMenuItem("Eyes Diverge",!Env.eyesCrossed);
		stereoViewDivergeMenuItem.addItemListener(this);
		stereoView.add(stereoViewDivergeMenuItem);
		
		m.add(stereoView);
		
		Menu quality3D = new Menu ("Quality");
		
		lowQualityMenuItem = new CheckboxMenuItem ("GPU?",false);
		lowQualityMenuItem.addItemListener(this);
		quality3D.add(lowQualityMenuItem);
		
		midQualityMenuItem = new CheckboxMenuItem ("GPU ",false);
		midQualityMenuItem.addItemListener(this);
		quality3D.add(midQualityMenuItem);
		
		highQualityMenuItem = new CheckboxMenuItem ("GPU$",false);
		highQualityMenuItem.addItemListener(this);
		quality3D.add(highQualityMenuItem);
		setQualitySelection();
		
		m.add(quality3D);
		
		m.addSeparator();
		
		Menu boxApp = new Menu ("Box");
		
		boxShadedMenuItem = new CheckboxMenuItem("Shaded",boxShaded);
		boxShadedMenuItem.addItemListener(this);
		boxApp.add(boxShadedMenuItem);

		bugWiredMenuItem = new CheckboxMenuItem("Wireframe",!boxShaded);
		bugWiredMenuItem.addItemListener(this);
		boxApp.add(bugWiredMenuItem);
		
		m.add(boxApp);
		
		Menu filApp = new Menu ("Filaments");
		
		filLinesMenuItem = new CheckboxMenuItem("Lines",Env.filamentLines);
		filLinesMenuItem.addItemListener(this);
		filApp.add(filLinesMenuItem);

		filCylsMenuItem = new CheckboxMenuItem("Cylinders",Env.filamentCyls);
		filCylsMenuItem.addItemListener(this);
		filApp.add(filCylsMenuItem);
		
		m.add(filApp);
		
		Menu monApp = new Menu ("Monomers");
		
		monSpheresMenuItem = new CheckboxMenuItem("Spheres",Env.eachMonomerASphere);
		monSpheresMenuItem.addItemListener(this);
		monApp.add(monSpheresMenuItem);
		
		monDotsMenuItem = new CheckboxMenuItem("Dots",Env.eachMonomerASphere);
		monDotsMenuItem.addItemListener(this);
		monApp.add(monDotsMenuItem);
		
		m.add(monApp);
		
		exampleMenuBar.add( m );

		// Add a menu to open frames allowing change of rates, concentrations, etc
		m = new Menu( "Change" );
		m.addActionListener(this);
		
		m.add("Run Stats");
		m.add("Initial Conditions");
		m.add("Rates");
		m.add("Concentrations");
		m.add("Mechanisms/Morphology");
		
		exampleMenuBar.add(m);
		
		exampleMenuBar.add(Info.infoMenu);
	
	}
	
	public void changeQualitySetting (int quality) {
		if (Env.graphicsQuality != quality) {
			synchronized(safeO) {
				Env.graphicsQuality = quality;
				setQualitySelection();
				Filament.eraseOldGraphics();
				updateBugScene();
			}
		}
	}
	
	public void setQualitySelection () {
		switch (Env.graphicsQuality) {
		case Env.LOW_QUALITY:
			lowQualityMenuItem.setState(true);
			midQualityMenuItem.setState(false);
			highQualityMenuItem.setState(false);
			break;
		case Env.MID_QUALITY:
			lowQualityMenuItem.setState(false);
			midQualityMenuItem.setState(true);
			highQualityMenuItem.setState(false);
			break;
		case Env.HIGH_QUALITY:
			lowQualityMenuItem.setState(false);
			midQualityMenuItem.setState(false);
			highQualityMenuItem.setState(true);
			break;
		}
	}


	//
	//  Handle checkboxes
	//
	public void itemStateChanged( ItemEvent event ) {
		Object src = event.getSource( );

		// Check if it is an appearance choice
		if ( src == boxShadedMenuItem ) {
			boxShadedMenuItem.setState(true);
			bugWiredMenuItem.setState(false);
			Thing.theCell.setFillAppearance();
			return;
		}
		
		if ( src == bugWiredMenuItem ) {
				bugWiredMenuItem.setState(true);
				boxShadedMenuItem.setState(false);
				Thing.theCell.setWireFrameAppearance();
				return;
		}
		
		if ( src == filLinesMenuItem ) {
			filLinesMenuItem.setState(true);
			filCylsMenuItem.setState(false);
			Env.filamentLines = true;
			Env.filamentCyls = false;
			Filament.eraseOldGraphics();
			synchronized(safeO) { updateBugScene(); }
			return;
		}
		
		if ( src == filCylsMenuItem ) {
			filCylsMenuItem.setState(true);
			filLinesMenuItem.setState(false);
			Env.filamentCyls = true;
			Env.filamentLines = false;
			Filament.eraseOldGraphics();
			synchronized(safeO) { updateBugScene(); }
			return;
		}
		
		if ( src == monSpheresMenuItem ) {
			monSpheresMenuItem.setState(true);
			monDotsMenuItem.setState(false);
			Env.eachMonomerASphere = true;
			Env.eachMonomerADot = false;
			Filament.eraseOldGraphics();
			synchronized(safeO) { updateBugScene(); }
			return;
		}
		
		if ( src == monDotsMenuItem ) {
			monSpheresMenuItem.setState(false);
			monDotsMenuItem.setState(true);
			Env.eachMonomerASphere = false;
			Env.eachMonomerADot = true;
			Filament.eraseOldGraphics();
			synchronized(safeO) { updateBugScene(); }
			return;
		}
		
		if ( src == paintOnMenuItem) {
			paintOn = paintOnMenuItem.getState();
		}
		
		if ( src == headlightMenuItem )
		{
			headlight.setEnable( headlightMenuItem.getState() );
		}
		
		if ( src == stereoViewOnMenuItem) {
			Env.stereoView = stereoViewOnMenuItem.getState();
			setCanvasForView();
		}
		
		if ( src == stereoViewCrossMenuItem) {
			Env.eyesCrossed = stereoViewCrossMenuItem.getState();
			stereoViewDivergeMenuItem.setState(!Env.eyesCrossed);
			if (Env.stereoView) { setCanvasForView(); }
		}
		
		if ( src == stereoViewDivergeMenuItem) {
			Env.eyesCrossed = !stereoViewCrossMenuItem.getState();
			stereoViewCrossMenuItem.setState(Env.eyesCrossed);
			if (Env.stereoView) { setCanvasForView(); }
		}
		
		if ( src == lowQualityMenuItem ) {
			changeQualitySetting(Env.LOW_QUALITY);
		}
		
		if ( src == midQualityMenuItem ) {
			changeQualitySetting(Env.MID_QUALITY);
		}
		
		if ( src == highQualityMenuItem ) {
			changeQualitySetting(Env.HIGH_QUALITY);
		}
		
		if (src == writeFileMenuItem) {
			if (writeFileMenuItem.getState()) { 
				if (Env.stereoView) {
					prepToStereoPairsFile();
				} else {
					prepToFile(); 
				}
			}
		}
		
		if (src == pausedMenuItem) {
			setPaused();	
		}
		
		if (src == runMenuItem) {
			setRunning();
		}
		
		if (src == rotateMenuItem) {
			viewRotation = rotateMenuItem.getState();
		}
		

		// Handle all other checkboxes
		super.itemStateChanged( event );
	}
	
	public void actionPerformed( ActionEvent event )
	{
		String arg = event.getActionCommand( );
		if (arg.equals("Redo")) { 
			setPaused();
			redoRun(); 
		}
		if (arg.equals("Reset View / Resize")) { resetView( ); }
		if (arg.equals("Run Stats")) { this.runControl.setVisible(true); }
		if (arg.equals("Rates")) { this.ratesControl.setVisible(true); }
		if (arg.equals("Concentrations")) { this.concControl.setVisible(true); }
		if (arg.equals("Initial Conditions")) { this.initControl.setVisible(true); }
		if (arg.equals("Mechanisms/Morphology")) { this.mechControl.setVisible(true); }
		
	}
	
	public void setRunning () {
		paused = false;
		pausedMenuItem.setState(false);
		runMenuItem.setState(true);
		runControl.setRunning();
	}
	
	public void setPaused () {
		paused = true;
		pausedMenuItem.setState(true);
		runMenuItem.setState(false);
		runControl.setPaused();
	}
	
	public static void allBranchGroupCapabilities(BranchGroup bg) {
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		bg.setCapability(BranchGroup.ALLOW_DETACH);
	}
	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.println(info);
	}
	
	public void prepToFile () {
		toFileName = null;
		toFileName = getFileToSave("Create directory for image sequence ...", this);
		if (toFileName == null) { 
			toFile = false;
			writeFileMenuItem.setState(false);
		} else {
			File saveLoc = new File(toFileName);
			saveLoc.mkdir();
			exCanvasL.setCaptureLoc(saveLoc.getAbsolutePath());
			toFile = true;
		}
	}
	
	public void prepToStereoPairsFile () {
		toFileName = null;
		toFileName = getFileToSave("Create directory for image sequences ...", this);
		if (toFileName == null) { 
			toFile = false;
			writeFileMenuItem.setState(false);
		} else {
			File saveLoc = new File(toFileName);
			saveLoc.mkdir();
			Env.leftImagePath = toFileName + File.separator + "leftEye";
			Env.rightImagePath = toFileName + File.separator + "rightEye";
			File leftI = new File(Env.leftImagePath);
			File rightI = new File(Env.rightImagePath);
			leftI.mkdir();
			rightI.mkdir();
			exCanvasL.setCaptureLoc(Env.leftImagePath);
			exCanvasR.setCaptureLoc(Env.rightImagePath);
			toFile = true;
		}
	}
	
	// File operations
	public static String getFileToSave (String dialogTitle, Component parent) {
		File file = null;
		String fileName = null;
		JFileChooser saveTo = new JFileChooser();
		saveTo.setDialogTitle(dialogTitle);
		saveTo.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int fileQuery = saveTo.showSaveDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = saveTo.getSelectedFile();
            fileName = file.getAbsolutePath();
            talkln (dialogTitle + fileName);
        } else {
            talkln ("Save cancelled by user");
            return null;
        }
        return fileName;
	}
}


