package actinScope;
/* 
 * Env.java ... herein are stored all the environmental variables and parameters changed by the gui
 */
/*  <ActinScope - for the simulation of dendritic actin networks>
    Copyright (C) <2005>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.text.*;
import java.awt.*;

import javax.vecmath.Color3f;

public class Env {
	// **** BASIC ****
	static final double degToRad = Math.PI/180;		// multiplier for conversion of degrees to radians
	static final double radToDeg = 180/Math.PI;
	
	// **** GRAPHICS AND FILE WRITING RELATED ****
	static double transScale = 5;		// just scales up all physical dimensions for a nice mapping in Java3D
	static int frameWidth = 1000;		//*** width of window frame in pixels.. not changeable during runtime in this code because of a bug in Apple's Java3D VM
	static int frameHeight = 500;		//*** height of window frame in pixels..  ditto above.
	static int canvasWidth = frameWidth/2;
	static int canvasHeight = frameHeight;
	static int toScreenInterval = 200;	// number of integration steps between drawing to screen
	static int toFileInterval = 4;	// number of integration steps between JPEGs written to file
	static boolean stereoView = false;	// flag for writing stereo pairs
	static boolean eyesCrossed = true;	// flag for eyes crossed in stereo view, as opposed to diverging
	static String leftImagePath;		// file path for storing left images
	static String rightImagePath;		// file path for storing right images
	
	// **** 3D Quality ****
	static int graphicsQuality = 0;	// specify graphics quality
	static final int LOW_QUALITY = 0;
	static final int	MID_QUALITY = 1;
	static final int HIGH_QUALITY = 2;
	
	static final int lowSphereTessalation = 10;	// number of polygons to define a sphere
	static final int midSphereTessalation = 18;
	static final int highSphereTessalation = 24;
	
	static final int lowCylXTessalation = 10;		// x and y resolution for cylinders
	static final int lowCylYTessalation = 10;
	static final int midCylXTessalation = 18;
	static final int midCylYTessalation = 18;
	static final int highCylXTessalation = 24;
	static final int highCylYTessalation = 24;
	
	// filament appearance
	static boolean filamentLines = true;
	static boolean filamentCyls = false;
	
	// monomer appearance
	static boolean eachMonomerASphere = false;
	static boolean eachMonomerADot = false;
	
	
	
	// **** ACTIN PROPERTIES ****
	static final double gActinDiameter = 0.0054;			//*** (m) diameter of G-actin
	static final double lengthPerActin = 0.5*gActinDiameter;	// (m) length added to filament by one actin monomer
	static final double helixWavelength = 0.037;			// wavelength of actin filament helix = 37 nm. (m)
	static final double actinPerTurn = helixWavelength/lengthPerActin; 	// number of monomers per helical turn
	static private final double branchAngle = 70;  			// mean branching angle.. from Pollard et al review 2000.
	static private final double branchAngleVar = 7;			// branch angle variance
	static final double widthToDiam = 0.007/0.0054;				// use measured values of f-actin & g-actin diameter to find ratio between
	static final double fActinDiameter = widthToDiam*gActinDiameter;				// diameter of actin filament
	static final double helixAngPerActin = Math.PI/actinPerTurn; // helical turn per actin monomer...only 1/2 actual turn per wavelength
	
	// **** THE "CELL" (BOX) SIZE ****
	static double cellXDim = 0.5;	// (microns) x dimension (axis along which surface translates) of cell
	static double cellYDim = 0.2;	// (microns) y dimension
	static double cellZDim = 0.2;	// (microns) z dimension
	static private double pistonStartPos = 3*gActinDiameter;	// initial position of the piston in actin diameters
	static boolean pistonJiggles = true;	// should the piston in cell have a random component to its position?
	static double pJigAmp = 0.1*gActinDiameter; // amplitude of the random jiggle
	
	// **** BRANCHING RELATED *****
	static private double branchZone = 3*gActinDiameter;				// (microns)zone near barbed end in which side-branches will be allowed
	static private double minARPgapLength = 3*gActinDiameter; 			// (microns)required min distance between branches
	static boolean branchOnlyAtSurface = true;			//*** does filament have to be near piston surface to branch?
	static double branchZoneFromSurface = 1;			//*** (x arpDiameter)...if branchOnlyAtSurface = true then branch within this many Arp2/3 diameters from surface
	
	// **** CAPPING RELATED *****
	static boolean surfaceBiasesCapping = true;			//*** does proximity to surface protect from capping?
	static double capBiasDistance = 1;					//*** (x capDiamter)...if surfaceBiasesCapping = true then protect within this many capping protein diameters
	
	// **** POLYMERIZATION RELATED ****
	static private final int initialSeeds = 100;					//*** initial number of filaments to nucleate
	static private final double minGapForPoly = 0.25*gActinDiameter;	//*** must be at least this much of a gap for a monomer to add to an existing filament
	
	// **** MASS OF PROTEINS ****  (kiloDaltons) kD
	static final double massOfActin = 42;
	static final double massOfAlphaActinin = 190;
	static final double massOfProfilin = 125;
	static final double massOfCap = 68;
	static final double massOfADF = 15;
	static final double massOfARP = 220;
	
	// **** EFFECTIVE RADII OF PROTEINS *** (m)
	// Use the following rules of thumb from Howard:
	// 		~1.2 nm^3 for each kD of protein (1.2e-9 m^3)
	//		~7 amino acids per nm^3
	// and assume globular so that volume = (4/3)*PI*r^3
	static final double reffCoeff = 3*(1.2e-9)/(4*Math.PI);
	static final double radOfActin = Math.pow(reffCoeff*massOfActin, 1.0/3.0);
	static final double radOfAlphaActinin = Math.pow(reffCoeff*massOfAlphaActinin, 1.0/3.0);
	static final double radOfProfilin = Math.pow(reffCoeff*massOfAlphaActinin, 1.0/3.0);
	static final double radOfCap = Math.pow(reffCoeff*massOfCap, 1.0/3.0);						//*** radius of globular capping protein
	static final double capDiameter = 2*radOfCap;	
	static final double radOfADF = Math.pow(reffCoeff*massOfADF, 1.0/3.0);
	static final double radOfARP = Math.pow(reffCoeff*massOfARP, 1.0/3.0);						//*** radius of globular Arp2/3 protein
	static final double arpDiameter = 2*radOfARP;
	
	//*** INITIAL CONCENTRATIONS FOR AMBIENT ENVIRONMENT **** (M)
	static private final double ACTIN_ATPConc = 0.0;			// initial molarity of actin monomers
	static private final double ACTIN_ADPConc = 0.0;			// initial molarity of actin monomers
	static private final double DIMERConc = 0.0;				// initial molarity of actin dimers
	static private final double TRIMERConc = 0.0;				// initial molarity of actin trimers
	static private final double WASPConc = 0.00;				// initial molarity of WASP/Scar proteins
	static private final double ARPConc = 0.0;						// initial molarity of ARP2/3
	static private final double ARPaConc = 1.0;						//*** initial molarity of active ARP2/3
	static private final double ADFConc = 3.0;				// initial molarity of actin depolymerizing factor ADF
	static private final double ADF_ATP_ACTINConc = 0.0;		// initial molarity of ADF bound ATP actin
	static private final double ADF_ADP_ACTINConc = 0.0;		// initial molarity of ADF bound ADP actin
	static private final double CAPConc = 2.0;						//*** initial molarity of capping proteins
	static private final double PROFILINConc = 2.50;			// initial molarity of the profilin
	static private final double PROFILIN_ATP_ACTINConc = 12;			//*** initial molarity of profilin bound ATP actin
	static private final double PROFILIN_ADP_ACTINConc = 0.0;	// initial molarity of profilin bound ADP actin
	static private final double ACTAConc = 0.00;				// initial molarity of ActA
	static private final double aACTININConc = 0.25;			// initial molarity of alpha-Actinin
	static private final double VASPConc = 0.50;				// initial molarity of VASP
	//************************************
	
	//	 **** REACTION RATES ****		_On (M^-1 s^-1)   _Off (s^-1)
	// capping protein
	static private final double kCap_On = 3.0;			//*** cap association rate 
	static private final double kCap_Off = 4.0e-4;		//*** cap dissociation rate 
		// barbed end dynamics
	static private final double kPlusATP_On = 11.6;		// plus-end association rate of ATP-actin 
	static private final double kPlusATP_Off = 1.4;		// plus-end dissociation rate of ATP-actin 
	static private final double kPlusADP_On = 3.8;		// plus-end association rate of ADP-actin 
	static private final double kPlusADP_Off = 7.2;		// plus-end dissociation rate of ADP-actin
	static private final double kPlusProfilin_On = 11.6;	//*** plus-end association rate of Profilin-ATP-actin 
	static private final double kPlusProfilin_Off = 1.4;	//*** plus-end dissociation rate of Profilin-ATP-actin 
		// pointed end dynamics
	static private final double kMinusATP_On = 1.3;		// minus-end association rate of ATP-actin 
	static private final double kMinusATP_Off = 0.8;	// minus-end dissociation rate of ATP-actin 
	static private final double kMinusADP_On = 0.16;	//*** minus-end association rate of ADP-actin 
	static private final double kMinusADP_Off = 5.0;//2.7;	//*** minus-end dissociation rate of ADP-actin 
		// profilin 
	static private final double kProfilinATP_On = 5.0;	// association rate of profilin and ATP-actin 
	static private final double kProfilinATP_Off = 5.0;	// dissociation rate of profilin and ATP-actin 
	static private final double kProfilinADP_On = 5.0;	// association rate of profilin and ADP-actin 
	static private final double kProfilinADP_Off = 10.0;	// dissociation rate of profilin and ADP-actin 
		// dimers and trimers
	static private final double kDimer_On = 2.0e-9;	// association rate of actin-actin to form a dimer 
	static private final double kDimer_Off = 5.0e-4;	// dissociation rate of dimer back to actin monomers 
		// arp2/3
	static private final double kBoundActAArp_Off = 0.6;	// (/s) same as kActAActinArp_Off below
	static private final double kArpFilamentNuc = 0.1;		// Check... rate of new filament nucleation by active ARP2/3
	static private final double kActAActin_On = 43.0;		// association rate of ActA with actin monomer 
	static private final double kActAActin_Off = 30.0;		// dissociation rate of ActA-actin
	static private final double kActAActinArp_On = 0.8;		// association rate of ActA-actin and Arp2/3
	static private final double kActAActinArp_Off = 0.6;	// dissociation rate of ActA-actin-Arp2/3
	static private final double kActAFactinArp_On = 0.8;	// association rate of ActA and Factin-Arp2/3
	static private final double kActAFactinArp_Off = 0.1;	// dissociation rate of ActA-Factin-Arp2/3
	static private final double kATPFactinArp_On = 2.0;		// association rate of Arp2/3 with F-actin (ATP)
	static private final double kATPFactinArp_Off = 0.1;	// dissociation rate of Arp2/3-Factin (ATP)
	static private final double kADPFactinArp_On = 2.0;		// association rate of Arp2/3 with F-actin (ADP)
	static private final double kADPFactinArp_Off = 2.0;	//*** dissociation rate of Arp2/3 with F-actin (ADP)
		// hydrolysis
	// **** vectorial model Carlier et al. 1987 and Melki et al. 1996 ****
	static private final double kATPtoADPPi = 12.3;				// hydrolysis rate of ATP to ADP-Pi (s^-1)
	static private final double kADPPitoADP = 0.01;//0.0026;	// dissociation of Pi from ADP-Pi  (s^-1)
		// adf/cofilin
	static private final double kATPFactinCofilin_On = 0.01;	// association rate of ATP-Factin with ADF/Cofilin
	static private final double kATPFactinCofilin_Off = 0.2;	// dissociation rate of ATP-Factin with ADF/Cofilin
	static private final double kADPFactinCofilin_On = 0.01;//0.1;	// association rate of ADP-Factin with ADF/Cofilin
	static private final double kADPFactinCofilin_Off = 0.05;	// dissociation rate of ADP-Factin with ADF/Cofilin
	static private final double kATPGactinCofilin_On = 0.01;	// association rate of ATP-Gactin with ADF/Cofilin
	static private final double kATPGactinCofilin_Off = 0.006; // dissociation rate of ATP-Gactin with ADF/Cofilin
	static private final double kADPGactinCofilin_On = 0.1;	// association rate of ADP-Gactin with ADF/Cofilin
	static private final double kADPGactinCofilin_Off = 0.012; // dissociation rate of ADP-Gactin with ADF/Cofilin
	static private final double kCofilinSever = 0.01;			// sever rate by Cofilin 
		// hack arp2/3 binding
	static private double kArpSideBranch = 5.0;				//*** side-branching hack rate constant
	
	// Parameters that will be user-changable
	// 		*** Rate Parameters ***
	static final String kOnUnits = " /microMolar-s";
	static final String kOffUnits = " /s";
	static final String concUnits = " microMolar";
	static final String distUnits = " microns";
	static final String degUnits = " degrees";
	static final String actinDiamUnits = " actin diameters (5.4 microns)";
	// actin dynamics
	static Parameter atpPlusOn = new Parameter (" ATP actin on rate - barbed end",kPlusATP_On,kOnUnits);
	static Parameter atpPlusOff = new Parameter (" ATP actin off rate - barbed end",kPlusATP_Off,kOffUnits);
	static Parameter adpMinusOn = new Parameter (" ADP actin on rate - minus end",kMinusADP_On,kOnUnits);
	static Parameter adpMinusOff = new Parameter (" ADP actin off rate - minus end",kMinusADP_Off,kOffUnits);
	// ARP2/3
	static Parameter adpFactinArpOff = new Parameter(" Arp2/3 off ADP f-actin",kADPFactinArp_Off,kOffUnits);
	static Parameter arpSideBranching = new Parameter(" Arp2/3 side-branching",kArpSideBranch,kOnUnits);
	// Capping Protein
	static Parameter capOn = new Parameter(" Capping protein on barbed end",kCap_On,kOnUnits);
	static Parameter capOff = new Parameter(" Capping protein off barbed end",kCap_Off,kOffUnits);
	//		*** Concentration Parameters ***
	static Parameter actinConc = new Parameter (" Profilin bound ATP g-actin",PROFILIN_ATP_ACTINConc,concUnits,false);
	static Parameter arpConc = new Parameter (" Arp2/3",ARPaConc,concUnits,false);
	static Parameter capConc = new Parameter (" Capping Protein",CAPConc,concUnits,false);
	//		*** Initial Condition Parameters ***
	static Parameter cellX = new Parameter (" Cell x-dimension",cellXDim,distUnits);
	static Parameter cellY = new Parameter (" Cell y-dimension",cellYDim,distUnits);
	static Parameter cellZ = new Parameter (" Cell z-dimension",cellZDim,distUnits);
	static Parameter initSeeds = new Parameter (" Initial actin seeds",initialSeeds," ",Parameter.INT);
	static Parameter pistDist = new Parameter (" Initial piston position", Env.pistonStartPos,distUnits);
	//		*** Mechanisms and Morphology ***
	static Parameter polyGap = new Parameter (" Minimum polymerization gap",Env.minGapForPoly,distUnits);
	static Parameter branchAng = new Parameter (" Mean branching angle",Env.branchAngle,degUnits);
	static Parameter branchAngVar = new Parameter (" Branch angle variation",Env.branchAngleVar,degUnits);
	static Parameter branchBarbedZone = new Parameter (" Branch region near barbed-end",Env.branchZone,distUnits);
	static Parameter branchMinDist = new Parameter (" Minimum distance between branches",Env.minARPgapLength,distUnits);
	
	// **** GUI AND COLOR RELATED ****
	static final Color3f universeColor3f = new Color3f(0.0f,0.0f,0.0f);
	static final Color universeColor = universeColor3f.get();
	static final Color3f cellColor3f = new Color3f(0.6f,0.6f,0.6f);
	static final Color cellColor = cellColor3f.get();
	static final Color cellShadeColor = Color.BLACK;
	static final Font controlFont = new Font(null,Font.PLAIN,9);
	static final Font headFont = new Font(null,Font.BOLD,10);
	static final Color controlBackColor = Color.LIGHT_GRAY;
	static final Color controlForeColor = Color.BLACK;
	static final Color controlFlashColor = Color.WHITE;
}
